/*
Klimalogger
Prototyp auf Lochraster
Alle 10s wird ein Wert per UART ausgegeben und jede Minute der Mittelwert gespeichert

Datum: 20.09.2007
Version 1.200

+ 1025 EEPROM vollstndig einbinden
+ Die Startzeit von LabVIEW beim Lschen in das interne EEPROM schreiben :-)
*/


#include <stdlib.h>
#include <avr/interrupt.h>
#include <i2cmaster.h>
#include <avr/sleep.h>
#include <util/delay.h>

#define BIT(x) (1<<(x))
#define SETBIT(p, b) (p) |=BIT(b)
#define CLRBIT(p, b) (p) &= ~BIT(b)
#define TOGGLE(p, b) (p) ^=BIT(b)
/*FuseBits:
Read: 	0x50 	0x00 	0xxx 	0xxx
Write: 	0xAC 	0XA0 	0x00 	0xCA
*/
#define F_CPU 2000000UL 		// 2 MHz
#define Dev24Cxx  0xA0   		//Adresse des EEPROM


unsigned int i, daten, baud, speicherplatz, wert, feuchte, temperatur, zaehler;
unsigned int Grenze=65534; //65534 fr ein 512 EEPROM, sonst wird es eine Endlosschleife!
unsigned int summe_feuchte, summe_temperatur; //Long passt leider nicht mehr :-(
char ausgabe[5];
unsigned char error,data,dummy, von_uart, jetzt, wert_low, wert_high, loeschen, speichern, dritter;

unsigned char sht_measure(unsigned int*, unsigned char*, unsigned char*);

//EEPROM schreiben Funktion
void schreiben(int adresse, unsigned char daten)
{
	unsigned char adr_high, adr_low;
	
	adr_low=adresse;
	adr_high=(adresse>>8);
    i2c_init();                                	// init I2C interface
	i2c_start_wait(Dev24Cxx+I2C_WRITE);       	// set device address and write mode
    i2c_write(adr_high);		                //Ergnzung auf 16bit Adresse
	i2c_write(adr_low);                       	// write address = 1
    i2c_write(daten);                      
    i2c_stop();                            		// set stop conditon = release bus
}
// Schreiben Ende

//EEPROM lesen Funktion
unsigned char lesen(unsigned int adresse)
{
	unsigned char adr_high, adr_low, les;
	
	adr_low=adresse;
	adr_high=(adresse>>8);
	i2c_init();                                	// init I2C interface
	i2c_start_wait(Dev24Cxx+I2C_WRITE);       	// set device address and write mode
	i2c_write(adr_high);
	i2c_write(adr_low);                     
	i2c_rep_start(Dev24Cxx+I2C_READ);       	// set device address and read mode
	les = i2c_readNak();                    	// read one byte
	i2c_stop();
	
	return (les);
}
//Lesen Ende


void seriell(char text[5],unsigned char mode)
{
	for (unsigned char k=0;k<5;k++)
	{
		data=text[k];							//kte-Stelle bernehmen
		text[k] = dummy;						//kte-Stelle lschen
		while (!(UCSRA& (1<<UDRE)));
		UDR=data;
	}
	while (!(UCSRA& (1<<UDRE)));
	if(mode==0) UDR = 0x0A;					//Linefeed
	else UDR = 0x09;							//Tabulator
}

void ablegen(unsigned int zahl)
{
	wert_low=(unsigned char) zahl;				//Den Integerbereich
	wert_high=(unsigned char) (zahl>>8);		//in Bytes bernehmen
	schreiben(speicherplatz,wert_high);
	++speicherplatz;
	schreiben(speicherplatz,wert_low);
	++speicherplatz;
}


SIGNAL(SIG_USART0_RX)     						//Handler fr UART Interrupt
{
	von_uart = UDR;								//Eingangspuffer lesen
}


SIGNAL(SIG_TIMER1_OVF)
{
	//TCNT1=3035;								//Das ergibt alle zwei Sekunden ein Interrupt bei Prescaler 64
	//TCNT1=26474;								//Alle 5s bei Prescaler 256
	TCNT1=46004;								//Alle 10s bei Prescaler 1024
	jetzt=1;
	speichern++;
}

void blink(unsigned char mal)
{
	for(unsigned char k=0;k<mal;k++)
	{
		CLRBIT(PORTD,6);
		_delay_ms(1);
		SETBIT(PORTD,6);
		_delay_ms(100);
		_delay_ms(100);
	}
}


int main(void)
{
	//Nur wenn der interne 4 MHz Taktgeber verwendet wird
	//CLKPR = 0b10000000;								//Clockprescaler Change-Enable
	//CLKPR = 0b00000001;								//Prescaler=2

	DDRB=0b00011111; 									//Festlegen der I/O-Pins
	DDRD=0b01111110;

	TCCR1B |= (1<<CS12) | !(1<<CS11) | (1<<CS10);		//Prescaler auf 64(0,1,1); 256(1,0,0); 1024(1,0,1)
	TIMSK |= (1<<TOIE1);								//Overflow Interrupt Enable
	
	set_sleep_mode(0);
	sleep_enable();
	
	baud=12;											//12=9600 bei 2MHz Takt, 25=4800
	UBRRH = (unsigned char)(baud>>8);
	UBRRL = (unsigned char) baud;
	/* Enable receiver and transmitter and interrupt */
	UCSRB = (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);
	/* Set frame format: 8data, 1stop bit */
	UCSRC = (1<<UCSZ1)|(1<<UCSZ0);

	blink(3);

	//EEPROM validieren
	while(!((wert_low==0xFF) && (wert_high==0x00)))	
	{
		wert_low=lesen(speicherplatz);
		++speicherplatz;
		wert_high=lesen(speicherplatz);
		++speicherplatz;
		if(speicherplatz>=Grenze)
		{
			speicherplatz=2;
			break;
		}
	}
	speicherplatz=speicherplatz-2;

	blink(6);
	
	sei();
	
	for(;;)
	{
		
		if(jetzt==1)
		{
			blink(1);
			
			error= sht_measure(&daten, &error, 0);
			temperatur=daten;
			summe_temperatur+= temperatur;
			
			error= sht_measure(&daten, &error, 1);
			feuchte=daten;
			summe_feuchte+= feuchte;
			
			utoa(temperatur,ausgabe,10);
			seriell(ausgabe,1);
			
			utoa(feuchte,ausgabe,10);
			seriell(ausgabe,1);
			
			utoa(speicherplatz,ausgabe,10);
			seriell(ausgabe,0);
			
			if(speichern==6)								//Nur den Mittelwert aus 6 Werten speichern
			{
				summe_temperatur=summe_temperatur/6;
				summe_feuchte=summe_feuchte/6;
				
				if(speicherplatz<(Grenze-8))				//Nicht mehr speichern wenn es zum berlauf kme
				{
					zaehler++;
					ablegen(zaehler);
					ablegen(summe_temperatur);
					ablegen(summe_feuchte);
					
					schreiben(speicherplatz,0xFF);			//EOF schreiben
					++speicherplatz;
					schreiben(speicherplatz,0x00);			//EOF schreiben
					speicherplatz=speicherplatz-1;
				}
				else blink(2);
				speichern=0;
				summe_temperatur=0;
				summe_feuchte=0;
			}
			jetzt=0;
		}	
		
		if (von_uart=='l')									//Lschen
		{
			speicherplatz=0;
			zaehler=0;
			schreiben(speicherplatz,0xFF);					//EOF schreiben
			++speicherplatz;
			schreiben(speicherplatz,0x00);					//EOF schreiben
			speicherplatz=speicherplatz-1;			
		}
		
		if (von_uart=='a')									//Dann erfolgt die Ausgabe
		{
			cli();											//Interrupte aus
			von_uart=dummy;
			utoa(speicherplatz,ausgabe,10);
			seriell(ausgabe,0);								//Datensatzlnge ausgeben
			
			for(int h=0; h<speicherplatz;)
			{
				CLRBIT(PORTD,6);
				von_uart=dummy;
				dritter++;
				wert= (unsigned int) ((lesen(h))<<8);
				wert+= (unsigned int)lesen(h+1);
				utoa(wert,ausgabe,10);
				if(dritter==3) {seriell(ausgabe,0); dritter=0;}
				else seriell(ausgabe,1);
				h+=2;
				SETBIT(PORTD,6);
			}
			sei();
		}
		sleep_cpu();
	}

}


